#ifndef LINKEDLIST_H
#define LINKEDLIST_H

template <class ItemType>
class linkedList {
	public:
		linkedList();
		~linkedList();
		linkedList(const linkedList& theList);
		linkedList<ItemType> & operator=(const linkedList& theList); 
		bool operator==(const linkedList& theList);
		void add(ItemType Item);
		int add(ItemType Item, int location);
		int remove(int location);
		int size();
		bool at(int location, ItemType & theItem);
		void reset();
		struct node {
			node* next;
			ItemType Item;
		};
	private:
		void copy(node*& tree1, node* const& tree2);
		bool compare(node*& tree1, node* const& tree2);
		int find(int location, node* tree, node*& spot);int find(int location, node*& spot);
		void addend(node*& tree, ItemType Item);	
		void kill(node*& tree);
		int size_;
		node* root;
};

template <class ItemType>
linkedList<ItemType>::linkedList(const linkedList& theList) {
    if(theList.root == NULL) {
	root = NULL;
	 return;
     }
    copy(root, theList.root);
}

template <class ItemType>
void linkedList<ItemType>::copy(node*& tree1, node* const& tree2) {
	if(tree2 == NULL) return;
	tree1 = new node;
	tree1->next = NULL;
	tree1->Item = tree2->Item;
	if(tree2->next != NULL) {
		copy(tree1->next, tree2->next);
	}
}

template <class ItemType>
linkedList<ItemType> & linkedList<ItemType>::operator=(const linkedList& theList) {
	kill(root);
	root = NULL;
	if(!(theList.root == NULL)) {
		copy(root, theList.root);
		return *this;
	} else return *this;
}
template <class ItemType>
linkedList<ItemType>::linkedList() {
	root  = NULL;
	size_ = 0;
}

template <class ItemType>
linkedList<ItemType>::~linkedList() {
	kill(root);
	root = NULL;
}
template <class ItemType>
int linkedList<ItemType>::size() {
	return size_;
}

template <class ItemType>
void linkedList<ItemType>::kill(node*& tree) {
	if(tree!=NULL) {
		kill(tree->next);
		delete tree;
	}
}

template <class ItemType>
void linkedList<ItemType>::reset() {
                size_ = 0;    
	kill(root);
	root = NULL;
}

template <class ItemType>
int linkedList<ItemType>::find(int location, node* tree, node*& spot) {
	if(tree==NULL) return 0;
	if(location!=0) {
		return(find(location - 1, tree->next, spot));
	} else {
		spot = tree;
		return 1;
	}
}

template <class ItemType>
void linkedList<ItemType>::addend(node*& tree, ItemType Item) {
	if(tree == NULL) {
		tree = new node;
		tree->Item = Item;
		tree->next = NULL;
	} else {
		addend(tree->next, Item);
	}
}

template <class ItemType>
void linkedList<ItemType>::add(ItemType Item) {
	addend(root, Item);
	size_++;
}

template <class ItemType>
int linkedList<ItemType>::add(ItemType Item, int location) {
	node* parent;
	node* nextspot;
	if(!find(location, root_, parent)) return 0;
	nextspot = parent->next;
	parent->next = new node;
	parent->next->next = nextspot;
	parent->next->Item = Item;
	return 1;
}

template <class ItemType>
int linkedList<ItemType>::remove(int location) {
	node* parent;
	node* child;
	if(location == 0) {
		if(root) {
			parent = root;
			root = root->next;
			return 1;
		} else {
			return 0;
		}
	}	
	if(!find(location-1, root, parent)) return 0;
	if(!find(location, root, child)) return 0;
	child = child->next;
	delete parent->next;
	parent->next = child;
	size_--;
	return 1;
}

template <class ItemType>
bool linkedList<ItemType>::at(int location, ItemType & theItem) {
	node* spot;
	if(!find(location, root, spot)) return FALSE;
	theItem = spot->Item;
	return (TRUE);
}

template <class ItemType>
bool linkedList<ItemType>::compare(node*& tree1, node* const& tree2) {
	if(tree1->Item == tree2->Item) {
		if(tree1->next == NULL && tree2->next == NULL) return TRUE;
			else if (tree1->next != NULL && tree2->next != NULL ) return(compare(tree1->next, tree2->next));
			else return FALSE;
	} else {
		return FALSE;
	}
}
template <class ItemType>
bool linkedList<ItemType>::operator==(const linkedList & theList) {
	if(size_ != theList.size_) return FALSE;
	return compare(root, theList.root);
}



#endif
