/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

//Name: openConnect()
//When&What Calls:  When user opens the connect dialog
//Purpose:  To show the window, and update the windows connection parameters
void kmpMain::openConnect()
{
    //Set the windows "Connection"
    connectWindow-> setConn(conn);
    //Show the window
    connectWindow -> show();
}

//Name:  returnStatus
//When&What Calls:  This is returned by the connection window
//Purpose:  To update the main windows connections, open and close connections
//	other then the statusConnection (done by connectionWindow) and change
//	the status of the client look (What buttons you can use ect...)
void kmpMain::returnStatus(int status, mpd_Connection *& conn_, QString ServerName, int ServerPort)
{
    conn = conn_;
    if(status) {
	//We stop trying to reconnect if we are....
	reconnectLoop->stop();
	//Inform the user that the client is now connected
	statusBar()->message("Connected to Server.");
	//Update the interface
	changeStatus(MPD_STATUS_STATE_STOP);
	//Re-Intialize Variables so that the a) playlist updates and b)Current song is Updated
	currentSongNumber  = -1;
	CurPlaylist = -1;
	//Save the server information (So we can open up additional Connections)
	serverName = ServerName;
	serverPort = ServerPort;
	//Give the playlist window the connection
	playlistWindow->setConn(conn);
	//Give the browse window the server information (So it can open up its own connection
	browseWindow->serverInfo(ServerName, ServerPort);
	unpauseStatus();
    } else {
	mainStatusLoop->stop();
	//*** This is when the connection has been closed ***
	//First close all the connections.
	if(conn != NULL) mpd_closeConnection(conn);
	conn = NULL;			//Command Connection
	browseWindow->closeConnection();	//Tell the browse window to close its connection
	
	//Now inform the user that we are no longer connected to the server
	statusBar()->message("Not Connected.");
	//Update the interface to unenabled
	changeStatus(MPD_STATUS_STATE_UNKNOWN);
    }
}

//Name:  changeStatus
//When&What Calls:  This is normally called by the Update Status
//Purpose:  To change the status of all of the widgets on the main form
void kmpMain::changeStatus(int status)
{
    //First we set all of the buttons to there respective status (enabled/unenabled)
    volumeBar->setEnabled(status);
    btn_prev->setEnabled(status);
    btn_stop->setEnabled(status);
    btn_PlayPause->setEnabled(status);
    btn_next->setEnabled(status);
    progressBar->setEnabled(status);
    randomL->setEnabled(status);
    repeatL->setEnabled(status);
    //And we pass it on to our children
    playlistWindow->enableMe(status);
    browseWindow->enableMe(status);
    //Now we store the status.
    serverStatus = status;
    //Now we see what the third button should be (play/pause)
    if(status == MPD_STATUS_STATE_PLAY) {
	//If we are playing, make the button PAUSE the player
	btn_PlayPause->setPixmap(pauseIcon);
    } else {
	//If we are not playing, make the button PLAY
	btn_PlayPause->setPixmap(playIcon);
    }
    //Finally we check to see if we are STOPPED (not paused)
    //	if we are we then set the current progress in the song to 0
    if(status == MPD_STATUS_STATE_STOP) {
	//Set the progress widget to 0 (beginning of a song)
	progressBar->setValue(0);
    }

}



//Name:  init()
//When&What Calls:  This is automatically called when the app starts
//Purpose:  To get KMP ready for all the kick ass things it will do.
void kmpMain::init()
{
    //**************MISC***************
    //Set the currentplaylist at 0 (so it updates on connect)
    CurPlaylist = 0;
    //Have the initial status bar message "Not Connected"
    statusBar()->message("Not Connected.");
    //These variables tell the client on status updates not to
    //update the respective widgets (So when dragging the indicator
    //it doesn't put it in the "Correct" spot
    progressPause = FALSE;
    volumePause = FALSE;
    //Set the initial connection to NULL (We are not connected)
    conn = NULL;   
    //Make the width of the bitrate the correct size for the current font
   // bitrate->setMaximumWidth(QFontMetrics(bitrate->font()).width("222"));
   // bitrate->setMinimumWidth(QFontMetrics(bitrate->font()).width("222"));
    atStatus = FALSE;
    QFont myFont;
    myFont.setBold(1);
    curTime->setFont(myFont);
    //**********************************
    
    //*************Set Main Window Sizes****
    int heightOfText = QFontMetrics(kbpsLabel->font()).height();
    mainBar->setGeometry(0,-1,318,heightOfText+4);
    mainBar->setMinimumSize(QSize(5, heightOfText+4));
    mainBarFrame->setMinimumSize(QSize(320,heightOfText+5));
    mainBarFrame->setGeometry(2,2,320,heightOfText+5);
    bitrate->setGeometry(5 , heightOfText+10 , QFontMetrics(bitrate->font()).width("222"), heightOfText);
    int curSpot = 5+ QFontMetrics(bitrate->font()).width("222");
    kbpsLabel->setGeometry(curSpot+3, heightOfText+10, QFontMetrics(kbpsLabel->font()).width("kbps"), heightOfText);
    curTime->setGeometry(321 - QFontMetrics(curTime->font()).width("-99:99"), heightOfText + 10, QFontMetrics(curTime->font()).width("-99:99"), heightOfText);
    
    curSpot+= QFontMetrics(kbpsLabel->font()).width("kbps") + 3;
    int space = (321 - QFontMetrics(curTime->font()).width("-99:99")) - curSpot;
    int padding = (space - 2* QFontMetrics(randomL->font()).width("random: off")) / 3;
    randomL->setGeometry(curSpot + padding , heightOfText +10, QFontMetrics(randomL->font()).width("random: off"), heightOfText);
    curSpot += QFontMetrics(randomL->font()).width("random: off") + padding;
    repeatL->setGeometry(curSpot + padding, heightOfText + 10, QFontMetrics(repeatL->font()).width("repeat: off"), heightOfText);
    LCD->setMinimumSize(QSize(325,2*heightOfText + 16));
    LCD->setMaximumSize(QSize(325,2*heightOfText + 16));   
    //*************************************
    
    
    
    //**********LOAD ICONS***********
    icons * MyIcons;
    MyIcons = new icons;
    playIcon = MyIcons->playI;
    pauseIcon = MyIcons->pauseI;
    random = 0;
    repeat = 0;
    delete MyIcons;
//    playIcon.load("pics/play.png");
//    pauseIcon.load("pics/pause.png");
    btn_PlayPause->setPixmap(playIcon);
    //**********************************
    
    //********WINDOW CREATION**********
    //We need to create a new instance for each of our windows
    //These are : Playlist Window, Browse Window, and Connect Window
    //And eventually will be: Options Window and About Window
    
    //**Playlist Window Creation***
    //Create a new playlist window
    playlistWindow = new kmpPlaylist;
    //Connect the increment playlist signal call to the increment playlist
    //	function on the main window
    connect(playlistWindow, SIGNAL(incrementPlaylist()), this, SLOT(incrementPlaylist()));
    //Connect the playlist closed signal call to the playlistClosed slot on
    //	main window
    connect(playlistWindow, SIGNAL(playlistClosed()), this, SLOT(playlistClosed()));
    //Connect the playlist connError signal call to the connError stlot on
    //	main window
    connect(playlistWindow, SIGNAL(connError()), this, SLOT(connError()));
    //And connect the playlist windows pause status and unpause status functions to the main windows
    connect(playlistWindow, SIGNAL(pauseStatus()), this, SLOT(pauseStatus()));
    connect(playlistWindow, SIGNAL(unpauseStatus()), this, SLOT(unpauseStatus())); 
    //And the playlist or the browse window isn't open yet
    playlistShown=FALSE;
    browseShown=FALSE;
    //**************************
    
    //**Browse Window Creation **
    //Create a new browse window
    browseWindow = new kmpBrowse;
    //Connect the browse closed signal call to the browseClosed slot on
    //	main window
    connect(browseWindow, SIGNAL(browseClosed()), this, SLOT(browseClosed()));
    connect(browseWindow, SIGNAL(connError()), this, SLOT(connError()));
    //**************************
    
    //**Connect Window Creation **
    //Create a new connect window
    connectWindow = new kmpConnect(this);
    //Connect the return status signal on the connect window to the
    //	respective one on the main window (to send server info)
    connect(connectWindow, SIGNAL( returnStatus(int, mpd_Connection*&, QString, int) ), this, SLOT(returnStatus(int, mpd_Connection *&, QString, int)) );
    //**************************
    
    //**Other Window Creation **
    //Create a new options window
    optionsWindow = new kmpOptions(this);
    connect(optionsWindow, SIGNAL(returnSettings(bool, QColor, QColor , int ) ) , this, SLOT(returnSettings( bool , QColor , QColor , int )));
    //Create a new about window
    aboutWindow = new kmpAbout(this);
    //**************************
    
    
    //*********************************
    
    
    //******* TIMER CREATION *********
    //The following section is for creating timers (in essence seperate threads)
    //Which are currently: mainStatusLoop, and scrollLoop
    
    //*****mainStatusLoop*****
    //Create the Timer
    mainStatusLoop = new QTimer(this);
    //Connect the timer timeout, to the statusloop slot 
    //This is basically updating the status in a seperate process
    connect(mainStatusLoop, SIGNAL(timeout()), this, SLOT(StatusLoop()));
    //************************
    
    //*******scrollLoop********
    //Create the Timer
    QTimer* scrollLoop = new QTimer(this);
    //Start the timer, with interval at 100ms (MIGHT ADD ABILITY TO CHANGE THIS)
    scrollLoop->start(100);
    //connect the scrollLoop timeout with the slot to move the main scrolling title
    //so that we will have a smooth scrolling title bar
    connect(scrollLoop, SIGNAL(timeout()), this, SLOT(ScrollIncrement()));
    //*************************
    
    //******reconnectLoop*****
    //Create the Timer
    reconnectLoop = new QTimer(this);
    //Connect the timer timeout to the reconnect Slot
    connect(reconnectLoop, SIGNAL(timeout()), this, SLOT(serverReconnect()));
    //***********************
    
    //**********************************
    
    
    //**************DnD****************
    //This section we give our children each other, so they can
    //Directly talk to each other :)
    playlistWindow->setBrowse(browseWindow);
    //**********************************
    
    
    //**************LOAD SAVED SETTINGS!
    loadSettings();
}


//Name:  Scroll Increment()
//When&What Calls:  This is called by the scrolltimer every 100ms
//Purpose:  To have the main title bar scroll when needed
void kmpMain::ScrollIncrement()
{
    //First we check if the text in the bar has changed
    if(ChangedMainBar) {
	//If the main bar has been changed, we must:
	//    1)reset the scrolling direction
	ScrollDirection = -1;
	//    2)reset the size of the bar to fit the text EXACTLY
	mainBar->setGeometry(2,-1,QFontMetrics(mainBar->font()).width(mainBar->text()), 20);
	//    3)reset the variable, so we don't keep fixing the main bar
	ChangedMainBar = FALSE;
    } else {
	//If we HAVEN'T changed the bar, we do the normal scrolling operations
	//  if we are at the end of the scroll, we pause for 2 seconds (20 100ms intervals)
	if(pauseScroll ) {
	    //Next we check to see if we have finished our pause
	    if(countForPause++ > 20) pauseScroll=FALSE;
	    //   if we haven't exit this function
	         else return;
	 }
	//Next we get the size of the main bar, so we know how big it is
	//	and thus how far we need to move it to see all of it
	QRect GetRect = mainBar->geometry();
	//If the size of the text is SMALLER then the size of the viewable area
	//	(around 317 pixels) then exit the function, no scrolling needed
	if(GetRect.width() < 317) return;
	//Otherwise set the new spot for the main bar
	GetRect.setLeft(GetRect.left() + ScrollDirection * 2);
	GetRect.setWidth(GetRect.width() + ScrollDirection * 2);
	//Then make these changes to the widget
	mainBar->setGeometry(GetRect);
	//Then we see if the we have reached an end
	if((GetRect.left() + GetRect.width() < 317) || GetRect.left() > 0) {
	    //If we have reached an end, start the scroll pause
	    countForPause = 0;
	    pauseScroll = TRUE;
	    //And change the direction of the scroll
	    ScrollDirection = ScrollDirection * (-1);
	}
    }
}
//Name:  StatusLoop
//When&What Calls: This is called by the statusTimer 
//Purpose:  To get the status from the server and update the client
void kmpMain::StatusLoop()
{
    atStatus = TRUE;
    //First off if the main connection is closed, we can't update!
    //	So we just retun
    checkForErrors(conn);
    if(conn==NULL) {
	atStatus = FALSE;
	return;
    }
    //Next we create a variable to hold the status of the server
    mpd_Status * status;
    //Then if there is nothing in the status,  there is an error!
    if((status = mpd_getStatus(conn))==NULL) {
	checkForErrors(conn);
	atStatus = FALSE;
	return;
    }
    
    //If the server does not have volume support, (unsupported mixer)
    //	we disable the volume bar
    if(status->volume == MPD_STATUS_NO_VOLUME) {
	volumeBar->setEnabled(false);
    } else {
	//If we do have volume support, update the volume.
	if(!volumePause) volumeBar->setValue(100 - status->volume);
    }
    //Next we check to see if the playlist in the server is different then
    //	what we think it is (CurPlaylist is incremented every time
    //	WE change the playlist, so if the server playlist has been
    //	updated more times then WE have changed it, we need to
    //	get the new playlist)
    if(CurPlaylist < status->playlist) {
	//We are now getting the new playlist, so store the new number
	CurPlaylist = status->playlist;
	statusBar()->message("Playlist Updating... "); 
	//Next we do a single shot timer (bascially spawn another
	//	process) to update the playlist. We do this here
	//	so all updates to the playlist are done in the
	//	background,  So there is no waiting to download
	//	the playlist when you open the playlist window!
	QTimer::singleShot( 50, this, SLOT(updatePlaylist()) );
    }
    if(random != status->random) {
	random = !random;
	if(random) randomL->setText("random: on");
	else  randomL->setText("random: off");
    }
    if(repeat != status->repeat) {
	repeat = !repeat;
	if(repeat) repeatL->setText("repeat: on");
	else  repeatL->setText("repeat: off");
    }
    
    //If the server is playing a song (or paused for that matter) We have alot more things to update
    if(status->state == MPD_STATUS_STATE_PLAY || status->state == MPD_STATUS_STATE_PAUSE){
	//We need to create some variables to hold the current song
	//	elapsed and total time. We store it in doubles so
	//	when we divide the two, we get a ratio, not a int.
	double elapsedTime = status->elapsedTime;
	double totalTime = currentSongLength = status->totalTime;
	//We need to check to see if the total time is 0, because dividing
	//	by zero is a big no no.
	if(totalTime != 0) {
	    //Also used to see if a song is loaded (or avaible to get info
	    //	time info on
	    //Now we get the current position of the status bar (which
	    //	has 1000 intervals on it, so we multiply the % 
	    //	complete of the song by 1000 to get which interval
	    //	it should be on)
	    double currentPos = 1000*elapsedTime/totalTime;
	    //Next we check to see if we should not update the widget
	    //	this is normally when the user is dragging on the position
	    //	of the song to seek.
	    if(!progressPause) progressBar->setValue((int)currentPos);
	    //Now we need to set the time ticker to the current time we are at
	    //	in the future you will be able to say if you want elapsed
	    //	time or time LEFT. I used a sub-function here, so that
	    //	at 9 seconds it would display 09, not 9.
	    if(elapsedTime == -1) elapsedTime =0;
	    if(negTime) {
		curTime->setText("-" + minTime((int)totalTime - (int)elapsedTime));
	    } else {
		curTime->setText(" " + minTime((int)elapsedTime));
	    }
	    //Then we update the bitrate. (As it can change through the song)
	    bitrate->setText(QString::number(status->bitRate));
	}
	//We need to tell the playlist window which song is playing so
	//	it somehow highlights it.
	playlistWindow->songPlaying(status->song);
	//Now we check to see if the current song playing is the song that WAS
	//	playing. (So when a new song plays we update information)
	if(currentSongNumber != status->song)  {
	    //If we have a new song, we need to get the new song information.
	    //So we create a entity to hold the information
	    mpd_InfoEntity * entity;
	    //And ask the server the information for this song number
	    //	using the COMMAND pipe.
	    mpd_sendPlaylistInfoCommand(conn,status->song);
	    //And of course check for errors in the process
	    checkForErrors(conn);
	    //Then we process the information
	    while((entity = mpd_getNextInfoEntity(conn))) {
		//If for some really wierd reason the information comming in the
		//	pipe isn't a song, we skip it, and hope we get what we
		//	want. This should NEVER happen, but b.s.t.s.
		if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
		    mpd_freeInfoEntity(entity);
		    continue;
		}
		//Then we create a new storable song (non pointers used)
		mpd_SongStore aSong;
		//And set all of the information to it.
		aSong.file = QString(entity->info.song->file);
		aSong.artist = QString(entity->info.song->artist);
		aSong.album = QString(entity->info.song->album);
		aSong.title = QString(entity->info.song->title);
		aSong.track = QString(entity->info.song->track);
		aSong.time = entity->info.song->time;
		//Then we update the Main bar's text using the title sub-function.
		if(showFilenameOnly) {
		    MainBarText = fileTitle(aSong.file);
		} else {
		    MainBarText = title(aSong);
		}
		//And update this text to the widget
		mainBar->setText(MainBarText);
		//And since we updated the playlist, we need to tell the scroll to
		//	update its information.
		ChangedMainBar = TRUE;
		//Now we release the memory that we used to get this new song's info
		mpd_freeInfoEntity(entity);
		//And we again check for errors. Just in case :-P
		checkForErrors(conn);
	    }
	    //Now that we have the new song's information we store that.
	    currentSongNumber = status->song;
	    //And we finish the last command given (playlist info)
	    mpd_finishCommand(conn);
	    //And check for errors!
	    checkForErrors(conn);
	} 
    //If the server is stopped, and the main bar has some text in it we change it to none
    //	This only happens when this is the first update after a stop.
    } else if (status->state == MPD_STATUS_STATE_STOP) {
	if((serverPort==0 && mainBar->text() != "" && conn==NULL) || conn != NULL) {
	    mainBar->setText("");
	    bitrate->setText("0");
	    curTime->setText(minTime(0));
	}
    }
    //Then we make SURE that we are currently at the same state as the server
    //	If it changed somewhere along the way, we update the interface
    if(serverStatus!=status->state) changeStatus(status->state);
    //Then we have to relase the memory holding the current status
    mpd_freeStatus(status);
    //And again check for errors!
    checkForErrors(conn);
    atStatus = FALSE;
}
		
//Name:  sendStop
//When&What Calls:  the Stop button
//Purpose:   To send a stop command to the server
void kmpMain::sendStop()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //If the player is ALREADY stopped, just return, the user is an idiot.
    if(serverStatus == MPD_STATUS_STATE_STOP) {
	unpauseStatus();
	return;
    }
    //If not we send the stop command.
    mpd_sendStopCommand(conn);
    //And check for errors of course
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //And check for errors again
    checkForErrors(conn);
    unpauseStatus();
}

//Name:  sendPlay
//When&What Calls:  the play/pause button
//Purpose:  to either send a play or pause command
void kmpMain::sendPlay()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    if(serverStatus == MPD_STATUS_STATE_PLAY) {
	//If we are currently playing, send a pause
	mpd_sendPauseCommand(conn);
    } else if (serverStatus == MPD_STATUS_STATE_PAUSE) {
	//If we are currently paused, send a play
	mpd_sendPauseCommand(conn);
    } else {
	//If we are currently stopped, then play from the beginning
	mpd_sendPlayCommand(conn, MPD_PLAY_AT_BEGINNING);
    }
    //And check for errors    
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //And check for errors again
    checkForErrors(conn);
    unpauseStatus();
}

//Name:  sendPrev
//When&What Calls:  the previous button
//Purpose:  to send a previous command to the server
void kmpMain::sendPrev()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //send the previous command to the server
    mpd_sendPrevCommand(conn);
    //And check for errors
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //And check for errors
    checkForErrors(conn);
    unpauseStatus();
}

//Name:  sendNext
//When&What Calls:  
//Purpose:  
void kmpMain::sendNext()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //send the next command to the server
    mpd_sendNextCommand(conn);
    //And check for errors
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //And check for errors    
    checkForErrors(conn);
    unpauseStatus();
}
    
//Name:  checkForErrors
//When&What Calls:  this is called a BAZILLION TIMES
//Purpose:  To check to see if there is any errors in the connection
//		we call this ALOT of times, because it doesn't
//		hurt to check, and its VERY usefull to check,
//		as this stops the client from crashing!
void kmpMain::checkForErrors(mpd_Connection *& conn_)
{
    //If this connection does NOT exist, we try to reconnect
    if(conn_ == NULL) {
	atStatus = FALSE;
	returnStatus(0, conn_, serverName, serverPort);
	reconnectLoop->start(3000);
	return;
    }
    //If there IS an error then we do some checks
    if(conn_->error) {
	//We create a string variable to hold the error
	QString Error;
	//And get the error
	Error = QString(conn_->errorStr);
	if(conn_->error == MPD_ERROR_ACK) {
	    //If the error is just an acknoledgement, (NON FATAL)
	    //	we just put it in the status bar for the user to see
	    statusBar()->message(Error.right(Error.length()-4));
	    //And clear the error so that everything will work correctly
	    mpd_clearError(conn_);
	    return;
	}
	//If we got here, then the error was fatal.... which is bad. So we disconnect.
	//	gracefully of course :-p
	mpd_clearError(conn_);
	//And we close the connection
	mpd_closeConnection(conn_);
	//And set it to NULL
	conn_ =NULL;
	returnStatus(0, conn_, serverName, serverPort);	
	//If the connection has been closed we have to reset the entire client...
	//And now we try to reconnect to the server every 3 seconds
	atStatus = FALSE;
	QTimer::singleShot( 1, this, SLOT(serverReconnect()) );
	reconnectLoop->start(3000);
    }
}
    
    
    
//Name:  pauseProgress()
//When&What Calls:  This is called when the user clicks DOWN on the progress bar
//Purpose:    To stop the progress bar from updating when the user is dragging
void kmpMain::pauseProgress()
{
    //When we set this variable to true, the status update skips the progress bar
    progressPause = TRUE;
}

//Name:  unPauseProgress()
//When&What Calls:  This is called when the client unclicks on the progress bar
//Purpose:  To send a seek command with the new position of the bar
void kmpMain::unpauseProgress()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //If we are in a stopped state, the user is stupid, so we just reset the bar to 0
    if(serverStatus == MPD_STATUS_STATE_STOP) {
	progressBar->setValue(0);
	return;
    }
    //Now we get from the widget, where we are at on it. (a value from 0-1000)
    double percentTime = progressBar->value();
    //And we divide it by 1000 to get the percent of the song we want to seek to
    percentTime/=1000;
    //Next we send the seek command at the percenttime * total seconds
    mpd_sendSeekCommand(conn, currentSongNumber, (int)(percentTime*currentSongLength));
    //And we check for Errors
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //And check for errors
    checkForErrors(conn);
    //And by setting this value to true again we are telling the status updates to
    //	resume updating the progress bar
    progressPause = FALSE;
    unpauseStatus();
}
    

//Name:  pauseVolume()
//When&What Calls:  This is called when the user Clicks DOWN on the volume bar
//Purpose:  To stop the status from updating the volume and get the intial volume
void kmpMain::pauseVolume()
{
    //by setting this variable to TRUE, we are telling the status to skip over volume when
    //	updating
    volumePause = TRUE;
    //We need to get the change in the volume, so we need a initial value.
    startVolume = volumeBar->value();
}

//Name:  unpauseVolume()
//When&What Calls:  this is called when the user lets up on the mouse button
//Purpose:  To send the change volume command to the server
void kmpMain::unpauseVolume()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //We send the server the change volume command
    mpd_sendVolumeCommand(conn, startVolume - volumeBar->value());
    //And check for errors
    checkForErrors(conn);
    //Then this command waits for the response from the server (To stop mis-
    //	communication through the pipe)
    mpd_finishCommand(conn);
    //Check for errors again
    checkForErrors(conn);
    //And by setting this variable to false, we are telling the status updates to resume
    //	updating the volume bar
    volumePause = FALSE;
    unpauseStatus();
}

//Name:  updatePlaylist
//When&What Calls:  This is called by the status checks when the playlist has changed
//Purpose:  To open a new connection download the new playlist, and then close that
//	additional connection. We decided to open another connection for this, as if
//	you are on a slow connection and have a huge playlist on a host far away, this
//	downloading of the playlist could take a significant amount of time. Which could
//	both a) slow down the client and b) hog up the pipe.   Since we don't want either of
//	these things to happen we open a connection dedicated to updating the playlist
void kmpMain::updatePlaylist()
{
    //We create some variables to hold the songs we get, the enitys we get, and the new connection
    mpd_SongStore aSong;
    mpd_Connection * playlistConn;
    mpd_InfoEntity * entity;
    //We open a new connection at the current server and port for the playlist
    playlistConn = mpd_newConnection (serverName, serverPort, 10);
    //Then we check to see if it opened fine
    checkForErrors(playlistConn);
    //If we didn't open fine, we stop trying to use this connection, and return from the function
    if(playlistConn == NULL) return;
    //Now we reset the linked list and unallocate all of its currently stored memory
    ServerList.reset();
    //And to get the playlist we send this command! using -1 to get the WHOLE list
    mpd_sendPlaylistInfoCommand(playlistConn, -1);
    //We loop through each enitity that we get0
    while((entity = mpd_getNextInfoEntity(playlistConn))) {
	//If for some reason we don't get a song we just skip it.
	// 	this should never happen but we do it anyways. b.s.t.s.
	if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
	    mpd_freeInfoEntity(entity);
	    continue;
	}
	//We set all of the information of the current song to a storable song type
	aSong.file = QString(entity->info.song->file);
	aSong.artist = QString(entity->info.song->artist);
	aSong.album = QString(entity->info.song->album);
	aSong.title = QString(entity->info.song->title);
	aSong.track = QString(entity->info.song->track);
	aSong.time = entity->info.song->time;
	//And then copy it to the linked list
	ServerList.add(aSong);
	//this command releases the memory used to store this song
	mpd_freeInfoEntity(entity);
	//And check for errors
	checkForErrors(playlistConn);
	//If there was an error the connection would be set to NULL
	//	so we exit the function if there was an error
	if(playlistConn==NULL) return;
    }
    //Now we send the end command, even though we are closing the connection
    //	Its just proper to do this.
    
    mpd_finishCommand(playlistConn);
    //After we have gotten every song, its time to close the connection
    mpd_closeConnection(playlistConn);
    //And we need to tell the playlist window to update the viewed playlist with this new one
    playlistWindow->populatePlaylist(ServerList);
    //And we tell the user that the playlist has been updated, and at what time.
    statusBar()->message("Playlist Updated: " + QTime::currentTime().toString());
}

//Name:  closeEvent
//When&What Calls: This is called when the main window has closed 
//Purpose:  At the moment it closes all of the windows, and de-allocates
//		all memory that we have allocated
void kmpMain::closeEvent( QCloseEvent * a) {
    //We close our two connections in this window
    if(conn != NULL) mpd_closeConnection(conn);
    conn = NULL;		 //Main Connection
    browseWindow->closeConnection(); //Tell the browse window to close its connection
    //Save Settings
    saveSettings();
    //Tell the other windows to close
    playlistWindow->close();
    connectWindow->close();
    browseWindow->close();
    optionsWindow->close();
    aboutWindow->close();
    //Delete the windows memory
    delete connectWindow;
    delete playlistWindow;
    //De-allocate memory for the linked list
    ServerList.reset();

    //And accept the close event
    a->accept();   
}

//Name:  showPlaylist
//When&What Calls: Playlist Button, or Menu Item
//Purpose:  To show/close the playlist window
void kmpMain::showPlaylist()
{
    //First we check to see if its already open, if so we close it
    if(playlistShown)  {
	playlistWindow->close();
    } else {
	//If its not open we show it, and then store that its open
	playlistWindow->show();
	playlistShown = TRUE;
	//And make sure the button is pushed down
	windowPlaylist->setOn(TRUE);
    }
}

//Name:  showBrowse
//When&What Calls: Browse Button, or Menu Item
//Purpose:  To show/close the Browse window
void kmpMain::showBrowse()
{
    //First we check to see if its already open, if so we close it
    if(browseShown)  {
	browseWindow->close();
    } else {
	//If its not open we show it, and then store that its open
	browseWindow->show();
	browseShown = TRUE;
	//And make sure the button is pushed down
	windowBrowse->setOn(TRUE);
    }
}

//Name:  increment playlist
//When&What Calls:  this is called by the playlist window
//Purpose:  to increment the internal playlist. So we don't update the playlist
//		we do ANYTHING to the playlist
void kmpMain::incrementPlaylist()
{
    CurPlaylist++;
}

//Name:  playlistClosed
//When&What Calls:  this is called by the playlistwindow when it closes
//Purpose:  To make sure the playlist button is un pressed and menu item unchecked
void kmpMain::playlistClosed()
{
    //We make sure the button is un "on"
    btn_pl->setOn(FALSE);
    //Store that the window is closed
    playlistShown = FALSE;
    //and make sure that the menu item is un "on"
    windowPlaylist->setOn(FALSE);
}

//Name:  browseClosed
//When&What Calls:  this is called by the browse window when it closes
//Purpose:  To make sure the browse button is un pressed and menu item unchecked
void kmpMain::browseClosed()
{
    //We make sure the button is un "on"
    btn_br->setOn(FALSE);
    //Store that the window is closed
    browseShown = FALSE;
    //and make sure that the menu item is un "on"
    windowBrowse->setOn(FALSE);
}

//Name:  showPlaylistM
//When&What Calls:  this is called by the menu item to open the playlist
//Purpose:  To open the playlist window from the menu
void kmpMain::showPlaylistM()
{
    //If the playlist is already shown, we hide it
    if(playlistShown)  {
	playlistWindow->close();
    } else {
	//If it isn't shown, then show it and make sure all the buttons are "on"
	playlistWindow->show();
	playlistShown = TRUE;
	windowPlaylist->setOn(TRUE);
	btn_pl->setOn(TRUE);
    }
}

//Name:  showbrowseM
//When&What Calls:  this is called by the menu item to open the browse window
//Purpose:  To open the browse window from the menu
void kmpMain::showBrowseM()
{
    //If the playlist is already shown, we hide it
    if(browseShown)  {
	browseWindow->close();
    } else {
	//If it isn't shown, then show it and make sure all the buttons are "on"
	browseWindow->show();
	browseShown = TRUE;
	windowBrowse->setOn(TRUE);
	btn_br->setOn(TRUE);
    }
}



//Name:  connError
//When&What Calls: this is called when any child window has connection problems  
//Purpose:  to close all the connections and start a reconnect
void kmpMain::connError()
{
    returnStatus(0, conn, serverName, serverPort);
    //And now we try to reconnect to the server every 3 seconds
    reconnectLoop->start(3000);
}

//Name:  serverReconnect
//When&What Calls:  this is called by the reconnect loop
//Purpose:  to reconnect kmp to the server
void kmpMain::serverReconnect()
{
    if(conn != NULL) {
	mpd_closeConnection(conn);
	conn = NULL;
    }
    if(serverName == "") { 
	reconnectLoop->stop();
	return;
    }
    statusBar()->message("Server Disconnected, Reconnecting...");
    conn = mpd_newConnection(serverName, serverPort,1);
    //Check to see if it succesfully connected
    if(conn->error) {
	//If we didn't close the connection and get ready to connect again
	mpd_closeConnection(conn);
	conn = NULL;
    } else {
	//Otherwise return that we have connected
	returnStatus(1, conn, serverName, serverPort);
    }
}


void kmpMain::pauseStatus()
{
    while(atStatus) {}
    mainStatusLoop->stop();
}


void kmpMain::unpauseStatus()
{
    mainStatusLoop->start(refreshInt);
}


void kmpMain::showOptions()
{
    optionsWindow->show();
}


void kmpMain::showAbout()
{
    aboutWindow->show();
}


void kmpMain::returnSettings( bool useFilename, QColor LCDbg_, QColor LCDfg_, int refreshRate )
{
    if(showFilenameOnly != useFilename) {
	showFilenameOnly = useFilename;
	playlistWindow->showFilename(useFilename);
	currentSongNumber = -1;
	CurPlaylist = -1;
    }
    LCD->setPaletteBackgroundColor(LCDbg_);
    LCD->setPaletteForegroundColor(LCDfg_);
    bitrate->setPaletteBackgroundColor(LCDbg_);
    bitrate->setPaletteForegroundColor(LCDfg_);
    kbpsLabel->setPaletteBackgroundColor(LCDbg_);
    kbpsLabel->setPaletteForegroundColor(LCDfg_);
    curTime->setPaletteBackgroundColor(LCDbg_);
    curTime->setPaletteForegroundColor(LCDfg_);
    mainBar->setPaletteBackgroundColor(LCDbg_);
    mainBar->setPaletteForegroundColor(LCDfg_);
    refreshInt = refreshRate;
    if(mainStatusLoop->isActive()) {
	pauseStatus();
	unpauseStatus();
    }
}


void kmpMain::saveSettings()
{
    QSettings settings;
    QString APP_KEY = "/kmp/";
    int r,b,g;
    
    //Main Window Settings
    settings.writeEntry( APP_KEY + "MainWindowX", x() );
    settings.writeEntry( APP_KEY + "MainWindowY", y() );
    r = LCD->paletteBackgroundColor().red();
    g = LCD->paletteBackgroundColor().green();
    b = LCD->paletteBackgroundColor().blue();
    settings.writeEntry( APP_KEY + "LCDbgR", r);
    settings.writeEntry( APP_KEY + "LCDbgG", g);
    settings.writeEntry( APP_KEY + "LCDbgB", b);
    r = LCD->paletteForegroundColor().red();
    g = LCD->paletteForegroundColor().green();
    b = LCD->paletteForegroundColor().blue();    
    settings.writeEntry( APP_KEY + "LCDfgR", r);
    settings.writeEntry( APP_KEY + "LCDfgG", g);
    settings.writeEntry( APP_KEY + "LCDfgB", b);
    settings.writeEntry( APP_KEY + "RefreshRate", refreshInt);
    settings.writeEntry( APP_KEY + "ShowFilename", showFilenameOnly);
    settings.writeEntry( APP_KEY + "NegativeTime", negTime);
    //Connect Window Settings
    QString ACname = connectWindow->ACname;
    int ACport = connectWindow->ACport;
    QStringList theList (connectWindow->stored);
    settings.writeEntry( APP_KEY + "ServerAutoName", ACname);
    settings.writeEntry( APP_KEY + "ServerAutoPort", ACport);
    settings.writeEntry( APP_KEY + "ServerList", theList);
    //PlaylistWindow
    settings.writeEntry( APP_KEY + "PLWindowX", playlistWindow->x() );
    settings.writeEntry( APP_KEY + "PLWindowY", playlistWindow->y() );
    settings.writeEntry( APP_KEY + "PLWindowWidth", playlistWindow->width() );
    settings.writeEntry( APP_KEY + "PLWindowHeight", playlistWindow->height() );
    //BrowseWindow
    settings.writeEntry( APP_KEY + "BRWindowX", browseWindow->x() );
    settings.writeEntry( APP_KEY + "BRWindowY", browseWindow->y() );
    settings.writeEntry( APP_KEY + "BRWindowWidth", browseWindow->width() );
    settings.writeEntry( APP_KEY + "BRWindowHeight", browseWindow->height() );
}
void kmpMain::loadSettings()
{
    QSettings settings;
    QString APP_KEY = "/kmp/";
    int r,b,g,x,y,w,h;
    QColor bg, fg;
    x = settings.readNumEntry( APP_KEY + "MainWindowX", 0 );
    y = settings.readNumEntry( APP_KEY + "MainWindowY", 0 );
    move( x, y );    
    r = settings.readNumEntry( APP_KEY + "LCDbgR", 178 );
    g = settings.readNumEntry( APP_KEY + "LCDbgG", 219 );
    b = settings.readNumEntry( APP_KEY + "LCDbgB", 223 );
    bg.setRgb(r,g,b);    
    r = settings.readNumEntry( APP_KEY + "LCDfgR", 0 );
    g = settings.readNumEntry( APP_KEY + "LCDfgG", 0 );
    b = settings.readNumEntry( APP_KEY + "LCDfgB", 0 );
    fg.setRgb(r,g,b);
    refreshInt = settings.readNumEntry( APP_KEY + "RefreshRate", 150 );
    showFilenameOnly = settings.readBoolEntry( APP_KEY + "ShowFilename", FALSE );    
    negTime = settings.readBoolEntry( APP_KEY + "NegativeTime", FALSE);
    //Propagate Settings
    returnSettings( showFilenameOnly, bg, fg, refreshInt );
    playlistWindow->showFilename(showFilenameOnly);
    //Pass to options Window
    optionsWindow->recieveOptions(showFilenameOnly, bg, fg, refreshInt);   
    //ConnectWindow
    QStringList theList;
    QString ACserver;
    int ACport;
    theList = settings.readListEntry( APP_KEY + "ServerList");
    ACserver = settings.readEntry( APP_KEY + "ServerAutoName", "");
    ACport = settings.readNumEntry( APP_KEY + "ServerAutoPort", 0);
    connectWindow->setList(theList);
    if(ACserver != "" && ACport != 0) {
	connectWindow->autoConnect(ACserver, ACport);
    }
    //PlaylistWindow
    x = settings.readNumEntry( APP_KEY + "PLWindowX", 0 );
    y = settings.readNumEntry( APP_KEY + "PLWindowY", 0 );
    w = settings.readNumEntry( APP_KEY + "PLWindowWidth", 400 );
    h = settings.readNumEntry( APP_KEY + "PLWindowHeight", 500 );
    playlistWindow->move(x,y);
    playlistWindow->resize(w,h);
    //BrowseWindow
    x = settings.readNumEntry( APP_KEY + "BRWindowX", 0 );
    y = settings.readNumEntry( APP_KEY + "BRWindowY", 0 );
    w = settings.readNumEntry( APP_KEY + "BRWindowWidth", 550 );
    h = settings.readNumEntry( APP_KEY + "BRWindowHeight", 575 );
    browseWindow->move(x,y);
    browseWindow->resize(w,h);
}



void kmpMain::toggleRandom()
{
    if(!conn) return;
    pauseStatus();
    mpd_sendRandomCommand(conn, !random);
    checkForErrors(conn);
    mpd_finishCommand(conn);
    checkForErrors(conn);
    random = !random;
    if(random) randomL->setText("random: on");
        else  randomL->setText("random: off");
    unpauseStatus();
}


void kmpMain::toggleRepeat()
{
    if(!conn) return;
    pauseStatus();
    mpd_sendRepeatCommand(conn, !repeat);
    checkForErrors(conn);
    mpd_finishCommand(conn);
    checkForErrors(conn);
    repeat = !repeat;
    if(repeat) repeatL->setText("repeat: on");
        else  repeatL->setText("repeat: off");
    unpauseStatus();
}


void kmpMain::toggleNegTime()
{
    negTime = !negTime;
}
