/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

//Name:  init
//When&What Calls:  initial intilization (called when loaded)
//Purpose:  To get the playlist manager window ready for user interaction
void kmpPlaylists::init() {
    //First we get the window so it resizes all the collums evenly when the widget resizes
    playlistsView->setResizeMode(QListView::AllColumns);
    //Next we create an accelerator so we can grab the key events
    QAccel * accel = new QAccel( this );
    //And we connect whenever there is a delete key even to the slot deletekeypressed
    accel->connectItem(accel->insertItem (Key_Delete), this, SLOT(deleteKeyPressed()));
}
    

//Name: setConn
//When&What Calls: this is called by the playlist window to give us the connection
//Purpose: To get the connection to send commands through
void kmpPlaylists::setConn( mpd_Connection * conn_)
{
    //We store our new connection
    conn = conn_;
    //And if there is a connection we update the list of playlists
    if(conn) refresh();
}
//Name: refresh
//When&What Calls: this is called when we get a new connection or the user hits refresh
//Purpose: To update the list of playlists available to load
void kmpPlaylists::refresh()
{
    //First we pause our status updates so we don't do 2 things at once
    pauseStatus();
    //Check to see if we have a connection
    if(!conn) {
	unpauseStatus();
	return;
    }
    //Clear our list of playlists
    playlistsView->clear();
    QListViewItem * ItemToAdd;
    mpd_InfoEntity * entity;
    //Send the command to the server to get the lists of playlists
    mpd_sendLsInfoCommand(conn, "");
    //And add each one to our list
    while((entity = mpd_getNextInfoEntity(conn))) {
	if(entity->type != MPD_INFO_ENTITY_TYPE_PLAYLISTFILE ) {
	    //This is a just in case measure
	    mpd_freeInfoEntity(entity);
	    continue;
	}
	if(entity->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE) {
	    ItemToAdd = new QListViewItem(playlistsView, QString(entity->info.playlistFile->path));
	}
	//We release the memory associated with the playlist sent to us
	mpd_freeInfoEntity(entity);
	
    }
    //And we resume status updates
    unpauseStatus();
}

//Name: selectItem
//When&What Calls: This is called when the user clicks on the tree widget
//Purpose: to put the name of the playlist in the edit box
void kmpPlaylists::selectItem( QListViewItem * theItem)
{
    //First we check if he clicked on an item, if not we return
    if(!theItem) return;
    //Then we set the text box to the name of the playlist
    nameEdit->setText(theItem->text(0));
}

//Name:  openItem
//When&What Calls: This is called when the user clicks open
//Purpose: to open a playlist
void kmpPlaylists::openItem()
{
    //First we pause the status updates
    pauseStatus();
    //Send the load command 
    mpd_sendLoadCommand(conn, nameEdit->text());
    checkForErrors();
    //Check for errors
    mpd_finishCommand(conn);
    checkForErrors(); 
    //And continue the status updates
    unpauseStatus();
}



//Name: openListItem
//When&What Calls: when the user double clicks on an item
//Purpose: to open a playlist from double clicking
void kmpPlaylists::openListItem( QListViewItem * theItem)
{
    //If they didn't click on a actual item we skip this
    if(!theItem) return;
    //Otherwise we set the text to the textbox and open the item
    nameEdit->setText(theItem->text(0));
    openItem();
}


//Name: checkForErrors
//When&What Calls: this is called serveral times to check for errors in the pipe
//Purpose:  We check for errors in the pipe to avoid kmp from crashing
void kmpPlaylists::checkForErrors()
{
    //If the pipe has an error...
    if(conn->error) {
	//Get the error
	QString Error;
	Error = QString(conn->errorStr);
	if(conn->error == MPD_ERROR_ACK) {
	    //If its just an acknoledgement, ignor it
	    mpd_clearError(conn);
	    return;
	} else {
	    //otherwise close the connection
	    mpd_clearError(conn);
	    mpd_closeConnection(conn);
	    conn =NULL;
                }
    }
}

//Name:  saveItem
//When&What Calls: This is called when the user clicks save
//Purpose: To save a playlist to the server
void kmpPlaylists::saveItem()
{
    //First we pause the status updates so we don't crash kmp
    pauseStatus();
    //Then we send the save command
    mpd_sendSaveCommand(conn, nameEdit->text());
    checkForErrors();
    //Check for errors
    mpd_finishCommand(conn);
    checkForErrors();
    //Allow status to update
    unpauseStatus();    
    //Refresh the list of playlists
    refresh();
}

//Name: deletekeypressed
//When&What Calls: this is called when the delete key is pressed
//Purpose:  to delete a playlist
void kmpPlaylists::deleteKeyPressed()
{
    //First we pause the status updates
    pauseStatus();
    QListViewItem * ItemAt;
    QListViewItem * ItemToDelete;
    //And we work our way from the bottom up
    ItemAt = playlistsView->lastItem();
    while(ItemAt != 0) {
	if(ItemAt->isSelected()) {
	    //And Delete each one that is selected
	    ItemToDelete = ItemAt;
	    mpd_sendRmCommand(conn, ItemToDelete->text(0));
	    checkForErrors();
	    //Check for errors
	    mpd_finishCommand(conn);
	    checkForErrors();
	    //Remove it from the widget
	    ItemAt = ItemAt->itemAbove();
	    playlistsView->takeItem(ItemToDelete);
	    continue;
	}
	//Move on
	ItemAt = ItemAt->itemAbove();
    }
    //And continue status updates
    unpauseStatus();
}

//Name: enableMe
//When& What Calls: This is called by playlist window
//Purpose: To change the status of the interface
void kmpPlaylists::enableMe( bool status )
{
    playlistsView->setEnabled(status);
    nameEdit->setEnabled(status);
    btn_save->setEnabled(status);
    btn_load->setEnabled(status);
    btn_refresh->setEnabled(status);
}
