/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/


void kmpPlaylist::init() {
    //First we set the playlist tree so it doesn't sort the songs (they need to be in the correct order)
    //	and to make sure that it doesn't do some funky resorting
    playlistTree->addColumn("#");
    playlistTree->addColumn("Title");
    playlistTree->addColumn("Time");
    playlistTree->setSorting(-1, TRUE);
    playlistTree->setSelectionMode(QListView::Extended);
    playlistTree->setResizeMode(QListView::NoColumn);
    playlistTree->setColumnWidthMode(1, QListView::Manual);
    playlistTree->setType(0);

    //Then we set the "total time" spot so that it fits the time perfecty so you can have up to
    //	999 hours of music with out cutting off the edges
    totalTimeLabel->setMaximumWidth(QFontMetrics(totalTimeLabel->font()).width(" 000:00:00 "));
    totalTimeLabel->setMinimumWidth(QFontMetrics(totalTimeLabel->font()).width(" 000:00:00 "));
    //And we load the current song playling icon here
    icons * MyIcons;
    MyIcons = new icons;
    playIcon = MyIcons->dotI;
    delete MyIcons;
    //Next we add an "accelerator" this enables the delete key so you can delete songs with it
    QAccel * accel = new QAccel( this );
    accel->connectItem(accel->insertItem (Key_Delete), this, SLOT(deleteKeyPressed()));
    //And we set the current song playling to -1 (So the indicator starts off correctly)
    curSong = -1;
    //And we don't want to have the current song playing until we get the playlist
    pausePaint  = TRUE;
    //And we create a new playlist manager window here
    managerWindow = new kmpPlaylists(this);
    connect(managerWindow, SIGNAL(pauseStatus()), this, SIGNAL(pauseStatus()));
    connect(managerWindow, SIGNAL(unpauseStatus()), this, SIGNAL(unpauseStatus()));
    //And we set the browsewindow to NULL
    browseWindow = NULL;
    playlistSize = 0;
    showOnlyFilename = FALSE;
}

//Name:  populatePlaylist
//When&What Calls:  this is called by the main window when it gets a new playlist
//Purpose:  To update the internal tree to show the playlist
void kmpPlaylist::populatePlaylist( linkedList<mpd_SongStore> & theList){
    playlistSize = 0;
    //We have a new playlist, so new time!
    totalTime = 0;
    //We store the list for later
    storedList = theList;
    //We make the tree not visible, so It updates lightning quick
    playlistTree->setHidden(TRUE);
    //And we get rid of the old playlist
    playlistTree->clear();
    //pausepaint is set to true so we don't try to say which song is playing before
    //	we even have the playlist updated
    pausePaint = TRUE;
    //Some temporary variables
    mpd_SongStore  theSong;
    QListViewItem  * ItemToAdd;
    //Next we loop through every song in the linked list, and put it into the tree
    for (int i = theList.size() - 1; i>=0; i--) {
	//Just in case there is no song at the spot we just skip it
	if(!theList.at(i, theSong)) continue;
	//For every song we add a new pointer attached to the tree with the songnumber, title and time
	if(showOnlyFilename) {
	    ItemToAdd = new QListViewItem(playlistTree, QString::number(i+1)+". ", fileTitle(theSong.file), minTime(theSong.time));
	}else {
	    ItemToAdd = new QListViewItem(playlistTree, QString::number(i+1)+". ", title(theSong), minTime(theSong.time));
	}
	playlistSize += 1;
	//And make sure that they can be dragged :)
	ItemToAdd->setDragEnabled(TRUE);
	if(theSong.time != -1) totalTime += theSong.time;
    }
    updatePlaylist();
    //Now that we have the playlist updated we can show it again
    playlistTree->setHidden(FALSE);
    //Right now we have no song highlighted so the "cur song" is -1
    curSong = -1;
    //And let the window paint the current song now that we have the new playlist
    pausePaint = FALSE;
    resetTime();
}

//Name:  playsong
//When&What Calls:  this is called when the user double clicks on a song
//Purpose:  to send a play command to the server of the current song playing
void kmpPlaylist::playsong( QListViewItem * theItem)
{
    pauseStatus();
    //Make sure we don't try to play a song if we aren't connected
    if(conn==NULL) return;
    //We don't really use the integer or the point, but we do stuff to em so we don't get an
    //And we send a playcommand at the current song, which we compute by where its at
    //	divided by how many pixels it is high, so we get the position this song is at
    mpd_sendPlayCommand(conn, theItem->itemPos() / theItem->height());
    //Then we do the usual check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();   
    unpauseStatus();
}
//Name:  checkForErrors
//When&What Calls:  this is called a bazillion times to check the pipe for errors
//Purpose:  To catch any errors and avoid crashes
void kmpPlaylist::checkForErrors()
{
    //If there is an error in our command connection
    if(conn->error) {
	//We get the error..
	QString Error;
	Error = QString(conn->errorStr);
	printf(Error);
	printf("\n");
	if(conn->error == MPD_ERROR_ACK) {
	    //If its just an acknoledgement error, we just clear it, as its not important
	    mpd_clearError(conn);
	    return;
	}
	//Otherwise we close the connection, and tell the main window to close the connections
	mpd_clearError(conn);
	mpd_closeConnection(conn);
	conn =NULL;
	//This tells the main window that we have had a connection error
	connError();
    }
}


//Name:  setConn
//When&What Calls:  this is called whenever the main window gets a new connection
//Purpose:  To give the playlist window the new command connection
void kmpPlaylist::setConn( mpd_Connection * conn_)
{
    conn = conn_;
    //And we pass the command connection to the manager window too.
    managerWindow->setConn(conn);
    //And we set the current song playling to -1 (So the indicator starts off correctly)
    curSong = -1;
    //And we don't want to have the current song playing until we get the playlist
    pausePaint  = TRUE;
}

//Name:  songPlaying
//When&What Calls:  this is called by the status checks and passes the current song playing
//Purpose:  To put an indicator next to the current song plaing
void kmpPlaylist::songPlaying( int songNum)
{
    //If the current song playing is already highlighted, skip this
    if (songNum == curSong)  return;
    //If we dont' have the full playlist yet, also skip this
    if(pausePaint) return;
    //Temporary pointer to a item
    QListViewItem  * ItemToEdit;    
    //If the song isn't the first one highlighted, then find the previous song highlighted and un highlight it
    if(curSong!= -1)  {
	//Find the last song highlighted and make no indicator on it
	findPointer(curSong, playlistTree, ItemToEdit);
	if(ItemToEdit) ItemToEdit->setPixmap(1, NULL);
    }
    //Now find the song that should have an indicator on it, and put it on it
    findPointer(songNum, playlistTree, ItemToEdit);
    ItemToEdit->setPixmap(1, playIcon);
    //And now we store that this song has the indicator
    curSong = songNum;
}

//Name:  findPointer
//When&What Calls:  this is called by functions to find the nth spot item in the tree
//Purpose:  to find an accurate nth spot item
void kmpPlaylist::findPointer( const int pos, QListView * theTree, QListViewItem * & theItem )
{
    //The first item is the first child of the tree
    theItem = theTree->firstChild();
    for(int i = 0; i < pos; i++) {
	//If there is no item we just return
	if(!theItem) return;
	//And we move down the list one item
	theItem = theItem->itemBelow();
    }
}

//Name:  deleteKeyPressed
//When&What Calls:  this is called when a user presses the delete key anywhere on the window
//Purpose:  To delete items from the playlist
void kmpPlaylist::deleteKeyPressed()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //Some temporary variables to store pointers
    mpd_SongStore theSong;
    QListViewItem * ItemAt;
    QListViewItem * ItemToDelete;
    //We start at the end... VERY important!
    ItemAt = playlistTree->lastItem();
    //We start a command List
    mpd_sendCommandListBegin(conn);
    //And check the pipe for errors    
    checkForErrors();
    //And we loop through EVERY item in the list
    while(ItemAt != 0) {
	//If the item is selected, then we delete it from the playlist tree
	if(ItemAt->isSelected()) {
	    ItemToDelete = ItemAt;
	    //We increment the playlist, because we are sending a command that will increment the
	    //	the server playlist. We do this so we don't get a new playlist everytime we delete
	    //	something
	    if(ItemToDelete->itemPos() / ItemToDelete->height() == curSong) curSong = -1;
	    incrementPlaylist();
	    //Finding the Time of the song in our linked list, then deleting the item from the list
	    if(storedList.at(ItemToDelete->itemPos() / ItemToDelete->height(), theSong)) {
		if(!storedList.remove(ItemToDelete->itemPos() / ItemToDelete->height())) {
		    unpauseStatus();
		    return;
		}
		//subtracting the time from total time
		totalTime-=theSong.time;
	    }
	    //displaying the new time
	    resetTime();
    	    //Sending the delete command
	    mpd_sendDeleteCommand(conn, ItemToDelete->itemPos() / ItemToDelete->height());
	    checkForErrors();
	    //And then move up the list
	    ItemAt = ItemAt->itemAbove();
	    //And get rid of the item in the widget tree
	    playlistTree->takeItem(ItemToDelete);
	    playlistSize--;
	    //And move on!
	    continue;
	}
	//If it isn't selected then we just move up!
	ItemAt = ItemAt->itemAbove();
    } 
    //End our command List
    mpd_sendCommandListEnd(conn);
    //Finish it all
    mpd_finishCommand(conn);
    //And check the pipe for errors
    checkForErrors();
    //After we delete a bunch of shit, we need to fix the numbers of the playlist items to be in order
    fixTree();
    unpauseStatus();
}

//Name:  fixTree
//When&What Calls:  this is called when the tree needs to be fixed
//Purpose:  to fix the numbers of the items so they are in order
void kmpPlaylist::fixTree()
{
    QListViewItem * ItemAt;
    //The first track to start on it 1
    int CurrentNumber = 1;
    //We get the first item (the first child of the tree)
    ItemAt = playlistTree->firstChild();
    //And loop through each one setting a new value for the track number
    while(ItemAt !=0) {
	ItemAt->setText(0, QString::number(CurrentNumber++)+". ");
	ItemAt = ItemAt->itemBelow();
    }
    updatePlaylist();
}

//Name:  closeEvent
//When&What Calls: this is called when the playlist gets the close event 
//Purpose:  To tell the main window that we have closed
void kmpPlaylist::closeEvent( QCloseEvent * a) {
    //This is connected to the main window and notifies it that we are closing
    playlistClosed();
    //Accept the close event
    a->accept();
}

//Name:  sendShuffle
//When&What Calls:  this is called when the user pressed the shuffle button
//Purpose:  To send the shuffle command
void kmpPlaylist::sendShuffle()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //Send the shuffle command to the server
    mpd_sendShuffleCommand(conn);
    //Check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();
    unpauseStatus();
}

//Name:  sendClear
//When&What Calls:  this is called when the user pressed the clear button
//Purpose:  To send the playlist clear command
void kmpPlaylist::sendClear()
{
    //During any commands sent through the pipe we have to pause the status update
    //	So we don't do 2 things at once :)
    pauseStatus();
    //Send the clear playlist command
    mpd_sendClearCommand(conn);
    //Check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();
    unpauseStatus();
}

//Name:  openManage
//When&What Calls: this is called when the user asks to open the playlist manager window 
//Purpose:  To show the playlist manager window
void kmpPlaylist::openManage()
{
    //Tell the manager window to show it self
    managerWindow->show();
}



//Name:  resetTime
//When&What Calls:  This is called when ever the total time is changed
//Purpose:  To reset the total time and the width of the label widget
void kmpPlaylist::resetTime()
{
    //Set the text to the new time
    totalTimeLabel->setText(hrTime(totalTime));
    //And make the label fit it perfectly
    totalTimeLabel->setMaximumWidth(QFontMetrics(totalTimeLabel->font()).width(hrTime(totalTime)));
    totalTimeLabel->setMinimumWidth(QFontMetrics(totalTimeLabel->font()).width(hrTime(totalTime)));
}

//Name: enableMe
//When&What Calls: this is called by the main window
//Purpose: To change the status of the interface
void kmpPlaylist::enableMe( bool status )
{
    //Change our status of our widgets
    playlistTree->setEnabled(status);
    btn_shuffle->setEnabled(status);
    btn_clear->setEnabled(status);
    totalTimeLabel->setEnabled(status);
    //And Pass it on to our children
    managerWindow->enableMe(status);
}



void kmpPlaylist::setBrowse( kmpBrowse * theWindow )
{
    browseWindow = theWindow;
}


void kmpPlaylist::addsong(QString MType, dragListView * source) {
    pauseStatus();
    tempList.reset();
    if(MType == "kmp_file") {
	mpd_SongStore theSong;
	QListViewItem * ItemAt;
	QListViewItem * ItemToAdd;
	ItemAt = source->firstChild();
	//We start a command List
	mpd_sendCommandListBegin(conn);
	//And check the pipe for errors    
	checkForErrors();
	//And we loop through EVERY item in the list
	while(ItemAt != 0) {
	    //If the item is selected, then we add it
	    if(ItemAt->isSelected()) {
		if(source->songList->at(ItemAt->itemPos() / ItemAt->height(), theSong)) {
		   
		    //We increment the playlist, because we are sending a command that will increment the
		    //	the server playlist. We do this so we don't get a new playlist everytime we delete
		    //	something
		    incrementPlaylist();
		    //add the time
		    totalTime+=theSong.time;
		    //displaying the new time
		    resetTime();
		    mpd_sendAddCommand(conn, theSong.file);
		    checkForErrors();
		    //And then move up the list
		    if(showOnlyFilename) {
			ItemToAdd = new QListViewItem(playlistTree, playlistTree->lastItem(), QString::number(++playlistSize) + ". " , fileTitle(theSong.file), minTime(theSong.time));
		    }else {
			ItemToAdd = new QListViewItem(playlistTree, playlistTree->lastItem(), QString::number(++playlistSize) + ". " , title(theSong), minTime(theSong.time));
		    }
		    ItemToAdd->setDragEnabled(TRUE);
		    ItemAt = ItemAt->itemBelow();
		    continue;
		}
	    }
	    //If it isn't selected then we just move up!
	    ItemAt = ItemAt->itemBelow();
	}
	mpd_sendCommandListEnd(conn);
	//Finish it all
	mpd_finishCommand(conn);
	//And check the pipe for errors
	checkForErrors();
    } else if(MType == "kmp_dir") {
	QListViewItemIterator it(source);
	QListViewItem * ItemToAdd;
	while (it.current()) {
	    if (it.current()->isSelected() ) {
		mpd_InfoEntity * entity;
		mpd_sendListallInfoCommand(conn, buildDirectoryPath(it.current()));
		while((entity = mpd_getNextInfoEntity(conn))) {
		//If for some really wierd reason the information comming in the
		//	pipe isn't a song, we skip it, and hope we get what we
		//	want. This should NEVER happen, but b.s.t.s.
		    if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
			mpd_freeInfoEntity(entity);
			continue;
		    }
		    //Then we create a new storable song (non pointers used)
		    mpd_SongStore aSong;
		    //And set all of the information to it.	    
		    aSong.file = QString(entity->info.song->file);
		    aSong.artist = QString(entity->info.song->artist);
		    aSong.album = QString(entity->info.song->album);
		    aSong.title = QString(entity->info.song->title);
		    aSong.track = QString(entity->info.song->track);
		    aSong.time = entity->info.song->time;
		    //And since we updated the playlist, we need to tell the scroll to
		    //	update its information.
		    //Now we release the memory that we used to get this new song's info
		    tempList.add(aSong);
		    mpd_freeInfoEntity(entity);
		    //And we again check for errors. Just in case :-P
		    checkForErrors();
		}
		mpd_sendCommandListBegin(conn);
		for(int i =0; i < tempList.size(); i++) {
		    mpd_SongStore aSong;
		    if(tempList.at(i, aSong)) {
			if(playlistSize == 4096) break;
			incrementPlaylist();
			//add the time
			totalTime+= aSong.time;
			//displaying the new time
			resetTime();
			mpd_sendAddCommand(conn, aSong.file);
			checkForErrors();
			if(showOnlyFilename) {
   			    ItemToAdd = new QListViewItem(playlistTree, playlistTree->lastItem(), QString::number(++playlistSize) + ". " , fileTitle(aSong.file), minTime(aSong.time));
			}else {
			    ItemToAdd = new QListViewItem(playlistTree, playlistTree->lastItem(), QString::number(++playlistSize) + ". " , title(aSong), minTime(aSong.time));
			}
			ItemToAdd->setDragEnabled(TRUE);
		    }
		}
		mpd_sendCommandListEnd(conn);
		//Finish it all
		mpd_finishCommand(conn);
		//And check the pipe for errors
		checkForErrors();
	    }
	++it;
	}
	
	
	
	
	
    }
    unpauseStatus();
    updatePlaylist();
}




void kmpPlaylist::showFilename( bool showFilename)
{
    showOnlyFilename = showFilename;
}



void kmpPlaylist::resizeE( QResizeEvent * e )
{
    int width = e->size().width();
    int height = e->size().height();
    if((playlistTree->lastItem()) && ((playlistTree->lastItem()->itemPos() + (playlistTree->lastItem()->height())) > (height - 25))) {
	playlistTree->setColumnWidth(1, width - playlistTree->columnWidth(0) - playlistTree->columnWidth(2)-19);
    } else {
	playlistTree->setColumnWidth(1, width - playlistTree->columnWidth(0) - playlistTree->columnWidth(2)-2);
    }
}


void kmpPlaylist::updatePlaylist()
{
    this->resize(this->width()+1, this->height()+1);
    this->resize(this->width()-1, this->height()-1);
}
